<?php

/*
WPDM Class:  List Table
Version:     1.0 
Modified:    20.07.2014
Last author: El_Panda
Notes:
*/


if(!class_exists('WP_List_Table')){
    require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

class List_Table extends WP_List_Table {

    var $args;

    function __construct($args){
        global $status, $page;
                
        $this->args = $args;  
                
        parent::__construct( array(
            'singular'  => $this->args['singular'],  
            'plural'    => $this->args['plural'],      
            'ajax'      => $this->args['ajax']        
        ) );
        
    }
    
    function column_default($item, $column_name){
        
        if (isset($this->args['column_'.$column_name]) && is_callable($this->args['column_'.$column_name])) {
            $value = isset($item[$column_name]) ? $item[$column_name] : null;
            return call_user_func($this->args['column_'.$column_name],$value,$item);
        } else {
            return $item[$column_name];
        }
        
    }
    
    function column_title($item){
        
        if (is_callable($this->args['column_title'])) {
            return call_user_func($this->args['column_title'],$item['title']);
        } else {
            return $item['title'];
        }
        
    }
    
    function column_cb($item){
        
        if (!isset($item['ID']) && isset($item['id'])) {
            $item['ID'] = $item['id'];
        }
        
        return sprintf(
            '<input type="checkbox" name="%1$s[]" value="%2$s" />',
            /*$1%s*/ $this->_args['singular'],  
            /*$2%s*/ $item['ID']                
        );
    }
    
    function get_columns(){
        
        if (is_callable($this->args['columns'])) {
            return call_user_func($this->args['columns']);
        } else {
            return $this->args['columns'];
        }
        
    }
    
    function get_sortable_columns() {
        
        if (is_callable($this->args['sortable_columns'])) {
            return call_user_func($this->args['sortable_columns']);
        } else {
            return $this->args['sortable_columns'];
        }
        
    }
    
    function get_bulk_actions() {
        
        if (is_callable($this->args['actions'])) {
            return call_user_func($this->args['actions']);
        } else {
            return $this->args['actions'];
        }
        
    }
    
    function process_bulk_action() {
        
        if (isset($this->args['process_bulk_action']) && is_callable($this->args['process_bulk_action'])) {
            return call_user_func($this->args['process_bulk_action'],$this->current_action());
        }
    
    }
    
    
    function prepare_items() {
        global $wpdb; 

        $per_page = $this->args['per_page'];
    
        $columns  = $this->get_columns();
        $hidden   = array();
        $sortable = $this->get_sortable_columns();
        
        $this->_column_headers = array($columns, $hidden, $sortable);
        
        $this->process_bulk_action();
        
        if (is_callable($this->args['data'])) {
            $data = call_user_func($this->args['data']);
        } else {
            $data = $this->args['data'];
        }
        
        if (!is_array($data)) { $data = array(); }
        
        if ((!empty($_REQUEST['orderby']))) {
            
            if (!function_exists('usort_reorder')) {
                function usort_reorder($a,$b){
                    $orderby = (!empty($_REQUEST['orderby'])) ? $_REQUEST['orderby'] : 'title';
                    $order = (!empty($_REQUEST['order'])) ? $_REQUEST['order'] : 'asc'; 
                    $result = strcmp($a[$orderby], $b[$orderby]); 
                    return ($order==='asc') ? $result : -$result; 
                }
            }
            usort($data, 'usort_reorder');
            
        }
        
        $current_page = $this->get_pagenum();
        $total_items  = count($data);
        
        if ($per_page!=-1) {
            $data = array_slice($data,(($current_page-1)*$per_page),$per_page);
        }
        
        $this->items  = $data;
    
        $this->set_pagination_args( array(
            'total_items' => $total_items,                  
            'per_page'    => $per_page,                     
            'total_pages' => ceil($total_items/$per_page)  
        ) );
    }

}

?>