<?php

/*
Plugin Name: Zoomable World Map
Plugin URI: https://www.fla-shop.com
Description: Interactive World Map with zoom, iPad/iPhone support, wide customization including pinpoint markers, landing pages, popups and colors. Extremely easy to use and to configure!
Version: 1.5.0
Author: Fla-shop.com
Author URI: https://www.fla-shop.com
License: 
*/

class Zoomable_Map_World {
    
    static private $instance = NULL;

    public $slug    = 'zoomable_map_world';
    public $name    = 'Zoomable Map - World';
    
    
    static public function getInstance() {
        if (self::$instance == NULL) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    
    function __construct() {
        
        if(session_id() == '') { session_start(); }
        
        register_activation_hook( __FILE__, array($this,'Install') );
        register_deactivation_hook( __FILE__, array($this,'Uninstall') );
        
        $this->plugin   = plugin_basename(__FILE__);
        $this->options  = $this->slug.'_options';
        $this->home     = get_option('home'); 
        $this->ajax_url = $this->home.'/index.php';
        
        $this->Init();

    }
    
    function Init() {
        
        $this->home = esc_url(home_url());
        
        add_action('wp_enqueue_scripts', array($this,'Enqueue'));
        add_action('admin_init',array($this,'Enqueue'));
        
        add_action('admin_menu', array($this,'Menu_add'));
        
        add_filter("plugin_action_links_$this->plugin", array($this,'Action_links') );
        
        add_action('init',array($this,'Init_hook'));
        
        add_shortcode( $this->slug, array($this,'shortcode') );
        
        $page = (isset($_REQUEST['page'])) ? $_REQUEST['page'] : null;
        
        if ( substr($page,0,strlen($this->slug)) == $this->slug ) {
           
            if ((isset($_REQUEST['action']) && $_REQUEST['action']=='export') or (isset($_REQUEST['action2']) && $_REQUEST['action2']=='export')) {
           
                if (isset($_REQUEST['map']) && count($_REQUEST['map'])>0) {
                    $this->Export('maps',$_REQUEST['map']);
                } else if (isset($_REQUEST['subject']) &&  count($_REQUEST['subject'])>0) {
                    $this->Export('subjects',$_REQUEST['subject']);
                } else if (isset($_REQUEST['point']) &&  count($_REQUEST['point'])>0) {
                    $this->Export('points',$_REQUEST['point']);
                }
                
                exit();
            
            }
        } 
        
    }
    
    function Init_hook() {
        
        if (isset($_REQUEST['ajax'])) {
            if (isset($_REQUEST['action']) && $_REQUEST['action']=='zoomable_map_action') {
                $this->Ajax();   
            }   
        }
        
    }
    
    function Install() {
        global $wpdb;
        
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        
        dbDelta("CREATE TABLE IF NOT EXISTS ".$wpdb->prefix.$this->slug."_maps (
                id int(11) NOT NULL AUTO_INCREMENT,
                name varchar(100) CHARACTER SET utf8 NOT NULL,
                width float NOT NULL,
                height float NOT NULL,
                borderColor varchar(20) CHARACTER SET utf8 NOT NULL,
                borderColorOver varchar(20) CHARACTER SET utf8 NOT NULL,
                backgroundColor varchar(20) CHARACTER SET utf8 NOT NULL,
                isResponsive int(1) NOT NULL,
                iOSbehavior int(1) NOT NULL,
                PRIMARY KEY (id)
              ) ENGINE=MyISAM DEFAULT CHARSET=utf8 AUTO_INCREMENT=1;");
        

        dbDelta("CREATE TABLE IF NOT EXISTS ".$wpdb->prefix.$this->slug."_subjects (
                id int(11) NOT NULL AUTO_INCREMENT,
                map_id int(11) NOT NULL,
                name varchar(100) CHARACTER SET utf8 NOT NULL,
                color varchar(20) CHARACTER SET utf8 NOT NULL,
                color2 varchar(20) CHARACTER SET utf8 NOT NULL,
                comment text NOT NULL,
                comment2 text NOT NULL,
                image varchar(100) NOT NULL,
                link varchar(100) NOT NULL,
                target varchar(10) NOT NULL,
                PRIMARY KEY (id)
              ) ENGINE=MyISAM DEFAULT CHARSET=utf8 AUTO_INCREMENT=1;");
        
        
        dbDelta("CREATE TABLE IF NOT EXISTS ".$wpdb->prefix.$this->slug."_points (
                id int(11) NOT NULL AUTO_INCREMENT,
                map_id int(11) NOT NULL,
                active int(1) NOT NULL,
                name varchar(100) CHARACTER SET utf8 NOT NULL,
                color varchar(20) CHARACTER SET utf8 NOT NULL,
                color2 varchar(20) CHARACTER SET utf8 NOT NULL,
                comment text NOT NULL,
                image varchar(100) NOT NULL,
                link varchar(100) NOT NULL,
                target varchar(10) NOT NULL,
                latitude varchar(20) NOT NULL,
                longitude varchar(20) NOT NULL,
                iconType varchar(20) NOT NULL,
                iconSize varchar(20) NOT NULL,
                PRIMARY KEY (id)
              ) ENGINE=MyISAM DEFAULT CHARSET=utf8 AUTO_INCREMENT=1;");
        
        
        $json = dirname(__FILE__).'/js/maps.json';
        if (file_exists($json)) $this->Import('maps','',$json);
    }
    
    function Uninstall() {
        global $wpdb;
        
        $wpdb->query("DROP TABLE ".$wpdb->prefix.$this->slug."_maps;");
        $wpdb->query("DROP TABLE ".$wpdb->prefix.$this->slug."_subjects;");
        $wpdb->query("DROP TABLE ".$wpdb->prefix.$this->slug."_points;");
        
    }
    
    function Enqueue() {
        
        $page = (isset($_REQUEST['page'])) ? $_REQUEST['page'] : null;
        
        if ( (is_admin() && ($page==$this->slug.'-maps' or $page==$this->slug.'-settings')) ) {
            
            $this->Enqueue_main(true,true);
            
        }
        
    }
    
    function Enqueue_main($is_admin,$in_footer=true) {
        
        // ### Styles
        
        // Flamap Styles
        wp_register_style($this->slug.'-leaflet', plugins_url('/js/leaflet.css', __FILE__));
        wp_enqueue_style($this->slug.'-leaflet');
        
        wp_register_style($this->slug.'-fa', plugins_url('/js/extensions/AwesomeIcons/css/font-awesome.min.css', __FILE__));
        wp_enqueue_style($this->slug.'-fa');
        
        wp_register_style($this->slug.'-flamap.awesome', plugins_url('/js/extensions/AwesomeIcons/css/flamap.awesome.css', __FILE__));
        wp_enqueue_style($this->slug.'-flamap.awesome');
        
        wp_register_style($this->slug.'-flamap', plugins_url('/js/flamap.css', __FILE__));
        wp_enqueue_style($this->slug.'-flamap');
        
        if ($is_admin) { 
        // Zoomable-map admin Styles
        wp_register_style($this->slug.'-admin', plugins_url('/css/admin.css', __FILE__));
        wp_enqueue_style($this->slug.'-admin');
        }
        
        // ### Scripts
        
        // jQuery, jQuery UI
        wp_enqueue_script('jquery');
        
        if ($is_admin) { 
        wp_enqueue_style('jquery-ui-css', plugins_url('/css/smoothness/jquery-ui.min.css', __FILE__));
        wp_enqueue_script('jquery-ui-core');
        wp_enqueue_script('jquery-ui-tabs');
        wp_enqueue_script('jquery-ui-autocomplete');
        wp_enqueue_script('jquery-ui-tooltip');
        wp_enqueue_script('jquery-ui-button');
        wp_enqueue_script('jquery-ui-tooltip');
        
        //Thickbox
        wp_enqueue_script('thickbox');
        wp_enqueue_style('thickbox', '/wp-content/includes/js/thickbox/thickbox.css');
        
        
        // Other
        wp_enqueue_script('farbtastic');
        wp_enqueue_style('farbtastic');
    
        wp_enqueue_script('tiny_mce');
        }
        
        
        // Zoomable-map core
        wp_register_script($this->slug.'-core', plugins_url('/js/core.js', __FILE__),array(),false,false);
        wp_enqueue_script($this->slug.'-core');
        
        
        if ($is_admin) {
        
        wp_register_script($this->slug.'-admin', plugins_url('/js/admin.js', __FILE__),array(),false,$in_footer);
        wp_enqueue_script($this->slug.'-admin');
    
            
        // YAML
        wp_register_script($this->slug.'-yaml', plugins_url('/js/yaml.js', __FILE__),array(),false,$in_footer);
        wp_enqueue_script($this->slug.'-yaml');
        
        
        }
        
        
        wp_register_script($this->slug.'-leaflet', plugins_url('/js/leaflet.js', __FILE__),array(),false,$in_footer);
        wp_enqueue_script($this->slug.'-leaflet');
        
        wp_register_script($this->slug.'-flamap', plugins_url('/js/flamap.js', __FILE__),array(),false,$in_footer);
        wp_enqueue_script($this->slug.'-flamap');
        
        wp_register_script($this->slug.'-flamap.data', plugins_url('/js/flamap.data.js', __FILE__),array(),false,$in_footer);
        wp_enqueue_script($this->slug.'-flamap.data');
        
        wp_register_script($this->slug.'-flamap.points', plugins_url('/js/extensions/points/flamap.points.js', __FILE__),array(),false,$in_footer);
        wp_enqueue_script($this->slug.'-flamap.points');
    
        wp_register_script($this->slug.'-flamap.awesome', plugins_url('/js/extensions/AwesomeIcons/flamap.awesome.js', __FILE__),array(),false,$in_footer);
        wp_enqueue_script($this->slug.'-flamap.awesome');
        
    }
    
    function Menu_add() {
        
        add_menu_page(__($this->name,$this->slug), __($this->name,$this->slug), 'manage_options', $this->slug.'-maps', array($this,'Menu') , 'dashicons-location-alt');
        
        add_submenu_page($this->slug.'-maps', __('Detailed settings',$this->slug), __('Detailed settings',$this->slug), 'manage_options', $this->slug.'-settings', array($this,'Menu'));
        
    }
    
    function Menu() {
        
        $page = trim(str_replace($this->slug,'',$_REQUEST['page']));
	$file = dirname(__FILE__).'/includes/page'.$page.'.php';
	
	if (file_exists($file)) {
            
	    include($file);
	}
        
    }
    
    function Action_links($links) {
        
        array_push($links, '<a href="admin.php?page='.$this->slug.'-settings">Settings</a>');
        return $links;
        
    }
    

    
    
    function Ajax() {
           
        switch($_REQUEST['subaction']) {
            case 'map_preview':
                
                @header("Content-type:text/html; charset='utf-8'");
                
                echo '<!DOCTYPE html>
                      <html>
                      <head>
                      <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />';
                
                wp_enqueue_script('jquery');       
                wp_head();
                
                echo '</head><body>';
                
                $this->Map('show',array(intval($_REQUEST['map_id'])));
                
                wp_footer();
                
                echo '</body></html>';
                
                break;
            case 'map_settings':
                
                $map_id  = intval($_REQUEST['map_id']);
                $tmp     = $this->Map('get',array($map_id));
                $Map     = array_shift($tmp);
                
                $Map->isResponsive = ($Map->isResponsive==1) ? true : false;
                $Map->iOSbehavior  = ($Map->iOSbehavior==1) ? true : false;
                
                if ($Map->isResponsive) {
                    unset($Map->width); unset($Map->height);
                }
                
                $results  = $this->Subject('get_by_map',array($map_id,'*'));
                $subjects = array(); $n=0;
                foreach($results as $result) {
                    
                    foreach($result as $rkey => $rvalue) { $result->{$rkey} = htmlspecialchars_decode($rvalue,ENT_QUOTES); }
                    
                    $n++; $subjects["state".$n] = $result;
                }
                
                $results = $this->Point('get_by_map',array($map_id,'*'));
                $points  = array(); $n=0;
                foreach($results as $result) {
                    if ($result->active==0) { continue; }
                    
                    foreach($result as $rkey => $rvalue) { $result->{$rkey} = htmlspecialchars_decode($rvalue,ENT_QUOTES); }
                    
                    $n++; $points["point".$n] = $result;
                }
                
                
                $options  = array(
                                 "options"  => $Map,
                                 "subjects" => $subjects,
                                 "points"   => $points,
                                 'ajax_url' => $this->ajax_url,
                                 );
                
                echo "Zmap.options = ".json_encode($options).";";
                
                break;
            case 'more_info':
                $subjects = $this->Subject('get',array(array(intval($_REQUEST['subject_id']))));
                $subject = array_shift($subjects);
                
                echo htmlspecialchars_decode($subject->comment2,ENT_QUOTES);
                
                break;
            case 'placeeditor_save':
                
                if (!current_user_can('manage_options')) { return false; }
                
                switch($_REQUEST['type']) {
                    case 'map':
                        
                        if ($_REQUEST['id']=='new') {
                            
                            $new_data = array(
                                              1 => array($_REQUEST['name'],
                                                    $_REQUEST['width'],
                                                    $_REQUEST['height'],
                                                    $_REQUEST['borderColor'],
                                                    $_REQUEST['borderColorOver'],
                                                    $_REQUEST['backgroundColor'],
                                                    $_REQUEST['isResponsive'],
                                                    $_REQUEST['iOSbehavior']));
                            
                            $this->Map('add',array($new_data,true));
                            
                        } else {
                        
                            $id = intval($_REQUEST['id']);
                            if ($id==0) { return false; }
                            
                            $new_data = array($id => $_REQUEST);
                            
                            $this->Map('edit',array($new_data));
                            
                        }
                        
                        break;
                    case 'subject':
                
                        $id = intval($_REQUEST['id']);
                        if ($id==0) { return false; }
                        
                        $new_data = array($id => $_REQUEST);
                        
                        $this->Subject('edit',array($new_data));
                        
                        break;
                    case 'point':
                        
                        if ($_REQUEST['id']=='new') {
                            
                            $new_data = array(
                                              1 => array($_REQUEST['map_id'],
                                                    $_REQUEST['active'],
                                                    $_REQUEST['name'],
                                                    $_REQUEST['color'],
                                                    $_REQUEST['color2'],
                                                    $_REQUEST['comment'],
                                                    $_REQUEST['image'],
                                                    $_REQUEST['link'],
                                                    $_REQUEST['target'],
                                                    $_REQUEST['latitude'],
                                                    $_REQUEST['longitude'],
                                                    $_REQUEST['iconType'],
                                                    $_REQUEST['iconSize']));
                            
                            $this->Point('add',array($new_data));
                            
                        } else { 
                        
                            $id = intval($_REQUEST['id']);
                            if ($id==0) { return false; }
                            
                            $new_data = array($id => $_REQUEST);
                            
                            $this->Point('edit',array($new_data));
                            
                        }
                        
                        break;
                }
                
                break;
        }
        
        exit();
    }
    
    public function Map($action,$data=array()) {
        
        $data = array_merge(array($this),$data);
        
        switch($action) {
            case 'show':
                
                return call_user_func_array(function($_this,$map_id,$echo=true) {
                    global $wpdb;

                    $_this->Enqueue_main(false,true);
                    
                    // Zoomable-map settings
                    wp_register_script($_this->slug.'-settings', $_this->ajax_url.'?ajax=1&action=zoomable_map_action&subaction=map_settings&map_id='.$map_id,array(),false,false);
                    wp_enqueue_script($_this->slug.'-settings');
                    
                    
                    $html =  '<!-- '.$_this->name.' -->
                          
                            <div id="flamap-holder" style="width:100%; height: 100%;">
                               <div id="map-'.$map_id.'" data-id="'.$map_id.'"></div>
                            </div>
                            
                            <script type="text/javascript">
                              jQuery(document).ready(function() {
                                  var map = new FLA.Map(\'map-'.$map_id.'\', \'flamap-holder\', Zmap.options.options, Zmap.options.subjects);
                                  map.loadPoints(Zmap.options.points);
                              });
                            </script>
                          
                          ';
                
                    if ($echo) {
                        echo $html;
                    } else {
                        return $html;
                    }
                
                },$data);
                
                break;
            case 'add':

                return call_user_func_array(function($_this,$data,$subjects=false) {
                    global $wpdb;

                    $query = "INSERT INTO ".$wpdb->prefix.$_this->slug."_maps (name,width,height,borderColor,borderColorOver,backgroundColor,isResponsive,iOSbehavior) VALUES ";
                    
                    $n=0;
                    foreach($data as $key => $value) {

                        if (count($value)<>8) { continue; }
                        
                        $n++; if ($n>1) {$query.=", ";}
                        
                        $query.="('".join("','",$value)."')";
                        
                    }
                    
                    $wpdb->query($query);
                    $map_id = $wpdb->get_var("SELECT LAST_INSERT_ID() FROM ".$wpdb->prefix.$_this->slug."_maps");
                    
                    // import default subjects
                    if ($subjects) { 
                        $json = dirname(__FILE__).'/js/maps.json';
                        if (file_exists($json)) {
                            
                            $hwnd = fopen($json,'r');
                            $data = fread($hwnd,filesize($json));
                            fclose($hwnd);
                            
                            for ($i=1;$i<3;$i++) { $data = htmlspecialchars_decode($data,ENT_QUOTES); }
                            $data = (array)json_decode($data);
                            $map  = array_shift($data);
                            
                            $_this->Import('subjects',$map_id,$map->subjects);
                        }
                    }
                    
                    return $map_id;
                
                },$data);
                
                break;
            
            case 'delete':
                
                return call_user_func_array(function($_this,$ids) {
                    global $wpdb;
                    
                    if (!is_array($ids)) { $ids = array($ids); }
                    
                    $wpdb->query("DELETE FROM ".$wpdb->prefix.$_this->slug."_maps WHERE id IN (".join(',',$ids).");");
                    
                },$data);
                
                break;
            
            case 'get':
                
                return call_user_func_array(function($_this,$ids=array()) {
                    global $wpdb;
                    
                    if (!is_array($ids)) { $ids = array($ids); }
                    
                    $query = "SELECT * FROM ".$wpdb->prefix.$_this->slug."_maps";
                    if (count($ids)>0) { $query.=" WHERE id IN (".join(',',$ids).")" ;}
                    
                    return $wpdb->get_results($query,OBJECT_K);
                    
                },$data);
                
                break;
            
            case 'edit':
                
                return call_user_func_array(function($_this,$data) {
                    global $wpdb;
                    
                    $map_ids        = array_keys($data);
                    $map_id         = array_shift($map_ids);
                    $data           = array_shift($data);
                    
                    foreach($data as $key => $value) {
                        $data[$key] = htmlspecialchars($value,ENT_QUOTES);
                    }
                    
                    $query          = "UPDATE ".$wpdb->prefix.$_this->slug."_maps SET
                    name            = '".$data['name']."',
                    width           = '".$data['width']."',
                    height          = '".$data['height']."',
                    borderColor     = '".$data['borderColor']."',
                    borderColorOver = '".$data['borderColorOver']."',
                    backgroundColor = '".$data['backgroundColor']."',
                    isResponsive    = '".$data['isResponsive']."',
                    iOSbehavior     = '".$data['iOSbehavior']."'
                    WHERE id        = $map_id;";
                    
                    $wpdb->query($query);
                    
                },$data);
                
                break;
        }
        
    }
    
    public function Subject($action,$data=array()) {
        
        $data = array_merge(array($this),$data);
        
        switch($action) {
            case 'add':

                return call_user_func_array(function($_this,$data) {
                    global $wpdb;

                    $query = "INSERT INTO ".$wpdb->prefix.$_this->slug."_subjects (map_id,name,color,color2,comment,comment2,image,link,target) VALUES ";
                    
                    $n=0;
                    foreach($data as $key => $value) {

                        if (count($value)<>9) { continue; }
                        
                        $n++; if ($n>1) {$query.=", ";}
                        
                        $query.="('".join("','",$value)."')";
                        
                    }
                    
                    $wpdb->query($query);
                    return $wpdb->get_var("SELECT LAST_INSERT_ID() FROM ".$wpdb->prefix.$_this->slug."_subjects");
                
                },$data);
                
                break;
            case 'get':
                
                return call_user_func_array(function($_this,$ids=array(),$fields='*') {
                    global $wpdb;
                    
                    $query = "SELECT $fields FROM ".$wpdb->prefix.$_this->slug."_subjects";
                    if (count($ids)>0) { $query.=" WHERE id IN (".join(',',$ids).") ORDER BY id" ;}
                    
                    return $wpdb->get_results($query,OBJECT_K);
                    
                },$data);
                
                break;
            
            case 'get_by_map':
            
                return call_user_func_array(function($_this,$map_id,$fields='id,name') {
                    global $wpdb;
                
                    $ids = $wpdb->get_results("SELECT id FROM ".$wpdb->prefix.$_this->slug."_subjects WHERE map_id=$map_id ORDER BY id;",OBJECT_K);
                    
                    if (count($ids)==0) { $ids = array(999999); }
                    
                    return $_this->Subject('get',array(array_keys($ids),$fields));
                    
                },$data);
                
                break;
            
            case 'edit':
                
                return call_user_func_array(function($_this,$data) {
                    global $wpdb;
                    
                    $subj_ids = array_keys($data);
                    $subj_id  = array_shift($subj_ids);
                    $data     = array_shift($data);
                    
                    foreach($data as $key => $value) {
                        $data[$key] = htmlspecialchars($value,ENT_QUOTES);
                    }
                    
                    $query    = "UPDATE ".$wpdb->prefix.$_this->slug."_subjects SET
                    name      = '".$data['name']."',
                    color     = '".$data['color']."',
                    color2    = '".$data['color2']."',
                    comment   = '".$data['comment']."',
                    comment2  = '".$data['comment2']."',
                    image     = '".$data['image']."',
                    link      = '".$data['link']."',
                    target    = '".$data['target']."'
                    WHERE id  = $subj_id;";
                    
                    $wpdb->query($query);
                    
                    $map_id = $wpdb->get_var("SELECT map_id FROM ".$wpdb->prefix.$_this->slug."_subjects WHERE id=$subj_id;");
                    
                    if (isset($_REQUEST['color_all']))  { $wpdb->query("UPDATE ".$wpdb->prefix.$_this->slug."_subjects SET color='".$data['color']."' WHERE map_id=$map_id;"); }
                    if (isset($_REQUEST['color2_all'])) { $wpdb->query("UPDATE ".$wpdb->prefix.$_this->slug."_subjects SET color2='".$data['color2']."' WHERE map_id=$map_id;"); }
                    
                },$data);
                
                break;
            
            case 'delete_by_map':
                
                return call_user_func_array(function($_this,$map_id) {
                    global $wpdb;
                    
                    if (!is_array($map_id)) { $map_id = array($map_id); }
                    
                    $wpdb->query("DELETE FROM ".$wpdb->prefix.$_this->slug."_subjects WHERE map_id IN (".join(',',$map_id).");");
                    
                },$data);
                
                break;
        }
        

        
    }

    
    
    public function Point($action,$data=array()) {
        
        $data = array_merge(array($this),$data);
        
        switch($action) {
            case 'add':

                return call_user_func_array(function($_this,$data) {
                    global $wpdb;

                    $query = "INSERT INTO ".$wpdb->prefix.$_this->slug."_points (map_id,active,name,color,color2,comment,image,link,target,latitude,longitude,iconType,iconSize) VALUES ";
                    
                    $n=0;
                    foreach($data as $key => $value) {

                        if (count($value)<>13) { continue; }
                        
                        $n++; if ($n>1) {$query.=", ";}
                        
                        $query.="('".join("','",$value)."')";
                        
                    }
                    
                    $wpdb->query($query);
                    return $wpdb->get_var("SELECT LAST_INSERT_ID() FROM ".$wpdb->prefix.$_this->slug."_points");
                
                },$data);
                
                break;
            
            case 'delete':
                
                return call_user_func_array(function($_this,$ids) {
                    global $wpdb;
                    
                    if (!is_array($ids)) { $ids = array($ids); }
                    
                    $wpdb->query("DELETE FROM ".$wpdb->prefix.$_this->slug."_points WHERE id IN (".join(',',$ids).");");
                    
                },$data);
                
                break;
            
            case 'get':
                
                return call_user_func_array(function($_this,$ids=array()) {
                    global $wpdb;
                    
                    $query = "SELECT * FROM ".$wpdb->prefix.$_this->slug."_points";
                    if (count($ids)>0) { $query.=" WHERE id IN (".join(',',$ids).")" ;}
                    
                    return $wpdb->get_results($query,OBJECT_K);
                    
                },$data);
                
                break;
            
            case 'get_by_map':
            
                return call_user_func_array(function($_this,$map_id,$fields='id,name') {
                    global $wpdb;
                
                    $ids = $wpdb->get_results("SELECT id FROM ".$wpdb->prefix.$_this->slug."_points WHERE map_id=$map_id;",OBJECT_K);
                    
                    if (count($ids)==0) { $ids = array(999999); }
                    
                    return $_this->Point('get',array(array_keys($ids),$fields));
                    
                },$data);
                
                break;
            
            case 'edit':
                
                return call_user_func_array(function($_this,$data) {
                    global $wpdb;
                    
                    $point_ids= array_keys($data);
                    $point_id = array_shift($point_ids);
                    $data     = array_shift($data);
                    
                    $query    = "UPDATE ".$wpdb->prefix.$_this->slug."_points SET
                    active    = '".$data['active']."',
                    name      = '".$data['name']."',
                    color     = '".$data['color']."',
                    color2    = '".$data['color2']."',
                    comment   = '".$data['comment']."',
                    image     = '".$data['image']."',
                    link      = '".$data['link']."',
                    target    = '".$data['target']."',
                    latitude  = '".$data['latitude']."',
                    longitude = '".$data['longitude']."',
                    iconType  = '".$data['iconType']."',
                    iconSize  = '".$data['iconSize']."'
                    WHERE id  = $point_id;";
                    
                    $wpdb->query($query);
                    
                    $map_id = $wpdb->get_var("SELECT map_id FROM ".$wpdb->prefix.$_this->slug."_points WHERE id=$point_id;");
                    
                    if (isset($_REQUEST['color_all']))  { $wpdb->query("UPDATE ".$wpdb->prefix.$_this->slug."_points SET color='".$data['color']."' WHERE map_id=$map_id;"); }
                    if (isset($_REQUEST['color2_all'])) { $wpdb->query("UPDATE ".$wpdb->prefix.$_this->slug."_points SET color2='".$data['color2']."' WHERE map_id=$map_id;"); }
                    
                    
                },$data);
                
                break;
            
            case 'delete_by_map':
                
                return call_user_func_array(function($_this,$map_id) {
                    global $wpdb;
                    
                    if (!is_array($map_id)) { $map_id = array($map_id); }
                    
                    $wpdb->query("DELETE FROM ".$wpdb->prefix.$_this->slug."_points WHERE map_id IN (".join(',',$map_id).");");
                    
                },$data);
                
                break;
        }
        
    }
    
    
    
    function Export($type,$ids) {
        
        switch($type) {
            case 'maps':
                $results = $this->Map('get',array($ids));
                
                foreach($results as $id => $result) {
                    $results[$id]->subjects = $this->Subject('get_by_map',array($id,'*'));
                    $results[$id]->points   = $this->Point('get_by_map',array($id,'*'));
                }
                
                break;
            case 'subjects':
                $results = $this->Subject('get',array($ids));
                break;
            case 'points':
                $results = $this->Point('get',array($ids));
                break;
        }
        
        if (count($results)>0) {
            
            $results = json_encode($results);
            
            header($_SERVER["SERVER_PROTOCOL"] . ' 200 OK');
            header('Content-Type: text/json');
            header('Content-Length: ' . (strlen($results)));
            header('Connection: close');
            header('Content-Disposition: attachment; filename="'.$type.'.json";');
            echo $results;
            
            exit();
        }
        
    }
    
    function Import($type,$map_id='',$data='') {
        
        if (is_string($data) && file_exists($data)) {
            
            $hwnd = fopen($data,'r');
            $data = fread($hwnd,filesize($data));
            fclose($hwnd);
            
            for ($i=1;$i<3;$i++) { $data = htmlspecialchars_decode($data,ENT_QUOTES); }
            $data = json_decode($data);
            
        } else {
            
            if (is_object($data)) {
                
                // ok
                
            } else { 
                
                $file = $this->slug.'_import_file';
                
                if(is_uploaded_file($_FILES[$file]["tmp_name"])) {
            
                    $hwnd = fopen($_FILES[$file]["tmp_name"],'r');
                    $data = fread($hwnd,filesize($_FILES[$file]["tmp_name"]));
                    fclose($hwnd);
    
                    @unlink($_FILES[$file]["tmp_name"]);
                    
                    for ($i=1;$i<3;$i++) { $data = htmlspecialchars_decode($data,ENT_QUOTES); }
                    $data = json_decode($data);
                    
                } else {
                    echo '<div class="error">'.__( 'File uploading error!', 'alabama-html5-map' ).'</div>';
                    return false;
                }
            
            }
            
        }
        
        switch($type) {
            case 'maps':
                
                foreach($data as $key => $value) {
                    
                    $map = array(
                                array(
                                    "name"            => htmlspecialchars($value->name,ENT_QUOTES),
                                    "width"           => htmlspecialchars($value->width,ENT_QUOTES),
                                    "height"          => htmlspecialchars($value->height,ENT_QUOTES),
                                    "borderColor"     => htmlspecialchars($value->borderColor,ENT_QUOTES),
                                    "borderColorOver" => htmlspecialchars($value->borderColorOver,ENT_QUOTES),
                                    "backgroundColor" => htmlspecialchars($value->backgroundColor,ENT_QUOTES),
                                    "isResponsive"    => htmlspecialchars($value->isResponsive,ENT_QUOTES),
                                    "iOSbehavior"     => htmlspecialchars($value->iOSbehavior,ENT_QUOTES), 
                                    ));
                    
                    $map_id = $this->Map('add',array($map));
                    
                    if (isset($value->subjects) and is_object($value->subjects) and count(get_object_vars($value->subjects))>0) {
                        $this->Import('subjects',$map_id,$value->subjects);
                    }
                    
                    if (isset($value->points) and is_object($value->points) and count(get_object_vars($value->points))>0) {
                        $this->Import('points',$map_id,$value->points);
                    }
                    
                }
                
                break;
            case 'subjects':
                
                $subjects = array();
                foreach($data as $key => $value) {
                    
                    $subjects[] = array(
                                        "map_id"   => $map_id,
                                        "name"     => htmlspecialchars($value->name,ENT_QUOTES),
                                        "color"    => htmlspecialchars($value->color,ENT_QUOTES),
                                        "color2"   => htmlspecialchars($value->color2,ENT_QUOTES),
                                        "comment"  => htmlspecialchars($value->comment,ENT_QUOTES),
                                        "comment2" => htmlspecialchars($value->comment2,ENT_QUOTES),
                                        "image"    => htmlspecialchars($value->image,ENT_QUOTES),
                                        "link"     => htmlspecialchars($value->link,ENT_QUOTES),
                                        "target"   => htmlspecialchars($value->target,ENT_QUOTES),
                                        );
                    
                }
                

                $this->Subject('add',array($subjects));
                
                break;
            case 'points':
                
                $points = array();
                foreach($data as $key => $value) {
                    
                    $points[] = array(
                                        "map_id"    => $map_id,
                                        "active"    => ((isset($value->active)) ? $value->active : 1),
                                        "name"      => htmlspecialchars($value->name,ENT_QUOTES),
                                        "color"     => htmlspecialchars($value->color,ENT_QUOTES),
                                        "color2"    => htmlspecialchars($value->color2,ENT_QUOTES),
                                        "comment"   => htmlspecialchars($value->comment,ENT_QUOTES),
                                        "image"     => htmlspecialchars($value->image,ENT_QUOTES),
                                        "link"      => htmlspecialchars($value->link,ENT_QUOTES),
                                        "target"    => htmlspecialchars($value->target,ENT_QUOTES),
                                        "latitude"  => htmlspecialchars($value->latitude,ENT_QUOTES),
                                        "longitude" => htmlspecialchars($value->longitude,ENT_QUOTES),
                                        "iconType"  => htmlspecialchars($value->iconType,ENT_QUOTES),
                                        "iconSize"  => htmlspecialchars($value->iconSize,ENT_QUOTES),
                                        );
                    
                }
                
                $this->Point('add',array($points));
                
                break;
        }
        
    }
    
    

    function invert_color($hex) {
        
        $hex = str_replace('#', '', $hex);
        
        $c_r = hexdec(substr($hex, 0, 2));
        $c_g = hexdec(substr($hex, 2, 2));
        $c_b = hexdec(substr($hex, 4, 2));
        $b   = (($c_r * 299) + ($c_g * 587) + ($c_b * 114)) / 1000;
        
        return ($b>130) ? '#000' : '#FFF';
        
    }
    function Shortcode($atts) {
        $map_id = intval($atts['id']);
        return $this->Map('show',array($map_id,false));
    }
}

$Zoomable_Map_World = Zoomable_Map_World::getInstance();

?>
